package global

import (
	"context"
	"enterprise/common/config"
	"enterprise/common/dao"
	"fmt"
	log "github.com/sirupsen/logrus"
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"gorm.io/gorm/logger"
	"time"
)

var (
	db *gorm.DB = nil
)

func GetDB() *gorm.DB {
	return db
}

type DBLogger struct {
	level     logger.LogLevel
	threshold int64
}

func (d *DBLogger) LogMode(level logger.LogLevel) logger.Interface {
	d.level = level
	return d
}

func (d *DBLogger) Info(context.Context, string, ...interface{}) {

}

func (d *DBLogger) Warn(context.Context, string, ...interface{}) {

}

func (d *DBLogger) Error(context.Context, string, ...interface{}) {

}

func (d *DBLogger) Trace(ctx context.Context, begin time.Time, fc func() (sql string, rowsAffected int64), err error) {
	sql, affects := fc()

	if err != nil && err != gorm.ErrRecordNotFound {
		log.Errorf("[SQL]sql=%s affect=%d cost=%dms error=%v", sql, affects, time.Since(begin).Milliseconds(), err)
	} else {
		if time.Since(begin).Milliseconds() > d.threshold {
			log.Errorf("[SQL]sql=%s affect=%d cost=%dms", sql, affects, time.Since(begin).Milliseconds())
		} else {
			log.Debugf("[SQL]sql=%s affect=%d cost=%dms", sql, affects, time.Since(begin).Milliseconds())
		}
	}
}

func initDB() error {
	cfg := config.GetConfig()
	/*
		   [mysql]
		   host = "10.0.2.157"
		   port = 3308
		   user = "root"
		   pass = "ZW5aaGVuMIIBIj"
		   db   = "enterprise"

		host = "14.22.113.49"
		port = 9352
	*/
	dsn := "root:ZW5aaGVuMIIBIj@tcp(10.0.2.157:3308)/enterprise"
	if config.IsDevEnv() {
		dsn = "root:ZW5aaGVuMIIBIj@tcp(14.22.113.49:9352)/enterprise"
	}

	var err error
	db, err = gorm.Open(mysql.Open(dsn), &gorm.Config{})
	if err != nil {
		log.Errorf("open dsn[%s] error[%s]", dsn, err)
		return err
	}
	db.Logger = &DBLogger{threshold: int64(2000)}
	dao.SetDB(db)

	//unify
	unifyDsn := fmt.Sprintf("%s:%s@tcp(%s:%d)/%s", cfg.UnifyMysql.User,
		cfg.UnifyMysql.Pass, cfg.UnifyMysql.Host, cfg.UnifyMysql.Port, cfg.UnifyMysql.Db)
	db, err = gorm.Open(mysql.Open(unifyDsn), &gorm.Config{})
	if err != nil {
		log.Errorf("open dsn[%s] error[%s]", dsn, err)
		return err
	}
	db.Logger = &DBLogger{threshold: int64(2000)}
	dao.SetUnifyDB(db)

	//corpy
	corpDsn := fmt.Sprintf("%s:%s@tcp(%s:%d)/%s", cfg.CorpMysql.User,
		cfg.CorpMysql.Pass, cfg.CorpMysql.Host, cfg.CorpMysql.Port, cfg.CorpMysql.Db)
	db, err = gorm.Open(mysql.Open(corpDsn), &gorm.Config{})
	if err != nil {
		log.Errorf("open dsn[%s] error[%s]", dsn, err)
		return err
	}
	db.Logger = &DBLogger{threshold: int64(2000)}
	dao.SetCorpDB(db)
	return nil
}

func DBPing() error {
	d, err := db.DB()
	if err != nil {
		return err
	}

	if err := d.Ping(); err != nil {
		return err
	}
	return nil
}
